@@ -85,9 +85,10 @@ editor_button_press_event_cb (GtkhtmlEditor *editor,
 	if (!in_selection) {
 		html_engine_disable_selection (html->engine);
 		html_engine_jump_at (html->engine, event->x, event->y);
-		gtkhtml_editor_update_context (editor);
 	}
 
+	gtkhtml_editor_update_context (editor);
+
 	gtk_menu_popup (
 		GTK_MENU (menu), NULL, NULL, NULL, NULL,
 		event->button, event->time);
@@ -101,6 +102,8 @@ editor_font_style_changed_cb (GtkhtmlEditor *editor,
 {
 	GtkHTMLFontStyle size;
 
+	editor->priv->ignore_style_change++;
+
 	gtk_toggle_action_set_active (
 		GTK_TOGGLE_ACTION (ACTION (BOLD)),
 		style & GTK_HTML_FONT_STYLE_BOLD);
@@ -127,6 +130,8 @@ editor_font_style_changed_cb (GtkhtmlEditor *editor,
 
 	gtk_radio_action_set_current_value (
 		GTK_RADIO_ACTION (ACTION (SIZE_PLUS_ZERO)), size);
+
+	editor->priv->ignore_style_change--;
 }
 
 static void
@@ -140,8 +145,12 @@ static void
 editor_paragraph_style_changed_cb (GtkhtmlEditor *editor,
                                    GtkHTMLParagraphStyle style)
 {
+	editor->priv->ignore_style_change++;
+
 	gtk_radio_action_set_current_value (
 		GTK_RADIO_ACTION (ACTION (STYLE_NORMAL)), style);
+
+	editor->priv->ignore_style_change--;
 }
 
 static void
@@ -1039,10 +1048,18 @@ gtkhtml_editor_set_changed (GtkhtmlEditor *editor,
 
 	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
 
-	html = gtkhtml_editor_get_html (editor);
+	if (!changed) {
+		html = gtkhtml_editor_get_html (editor);
 
-	if (!changed)
-		html_engine_saved (html->engine);
+		/* XXX The NULL check was added to deal with a race in
+		 *     Evolution when saving a message to a remote Drafts
+		 *     folder (which is asynchronous) and then closing the
+		 *     editor (which is immediate).  The GtkHTML object
+		 *     may already be disposed by the time the Save Draft
+		 *     callback gets here.  See bug #553995 for details. */
+		if (html)
+			html_engine_saved (html->engine);
+	}
 
 	editor->priv->changed = changed;
 }
